import os
import pandas as pd
import json
from .database.mycosmosdb import callConnect
from .endpoints.utils import read_item
from .endpoints.measurements import post_custom_measure
from .endpoints.measurements import PostData

# this script will import the data from: metric-api/app/database/EIP Historical Impact Data.xlsx
# to use it: python -m app.ImportImpactData

# Set up the path for the Excel file
current_directory = os.path.dirname(os.path.realpath(__file__))
excel_file_path = os.path.join(current_directory, 'database', 'EIP Historical Impact Data.xlsx')

# Read the Excel file into a pandas DataFrame
# number of rows limited for testing
# remove this for actual implementation
df = pd.read_excel(excel_file_path, nrows=12)
df = df.fillna("")
df = df.astype(str)

def find_or_create_section(data, label, structure):
    """
    Recursively finds or creates a section with the given label in the data structure.
    """
    for item in data:
        if item.get('label') == label:
            return item

    # If not found, create a new section with the specified structure
    new_section = {'label': label, **structure}
    data.append(new_section)
    return new_section

def update_json_value(json_data, df_row, json_field, df_field):
    # Convert the DataFrame value to a string
    value = df_row[df_field]

    # Ensure the 'custommeasure' section exists
    custom_measure = json_data.setdefault('custommeasure', [])

    # Find or create the 'ImpactOverview' section
    impact_overview = find_or_create_section(custom_measure, 'ImpactOverview', {'metrics': []})

    # Find or create the '2022' metric
    metric_2022 = find_or_create_section(impact_overview['metrics'], '2022', {'value': []})

    # Find or create the 'Q0' value data
    value_data_q0 = find_or_create_section(metric_2022['value'], 'Q0', {'metrics': []})

    # Find or create the '1' q0 metric
    q0_metric_1 = find_or_create_section(value_data_q0['metrics'], '1', {'value': [{}]})

    # Update the value in the JSON data
    q0_metric_1['value'][0][json_field] = value

    # return all the responses
    return q0_metric_1['value'][0]


# Function to update JSON data based on DataFrame row
async def update_json_from_df(data, df_row):
    # Mapping of DataFrame columns to JSON fields
    field_mapping = {
        'EIP Coalition Customers CY20': 'coalitionCY20',
        'EIP Coalition Customers CY21': 'coalitionCY21',
        'EIP Coalition Customers CY22': 'coalitionCY22',
        'Energy Customers CY20': 'energyCY20',
        'Energy Customers CY21': 'energyCY21',
        'Energy Customers CY22': 'energyCY22',
        'Total Customers CY20': 'customersCY20',
        'Total Customers CY21': 'customersCY21',
        'Total Customers CY22': 'customersCY22',
        'Other Non-financial KPIs': 'otherKPIs'
    }

    d = {}
    # Update each field in the JSON data based on the mapping
    for df_field, json_field in field_mapping.items():
        d = update_json_value(data, df_row, json_field, df_field)
    
    print(f"data: ")
    await post_custom_measure(PostData(
        oid= data['id'],
        email= data['email'],
        year= "2022",
        quarter= "Q0",
        metric= 'ImpactOverview',
        data= d
    )) 

# Main code
async def main():
    for index, row in df.iterrows():

        # Retrieve the JSON data from the database for each email
        oid = row['OID']
        # conditional for testing with dragonfly energy company
        # remove this conditional for actual implementation
        if oid is None or oid != '60d9f613-565c-42c3-9e94-d9fe8e128cc1':
            print("Skipping iteration due to empty 'oid'")
            continue  # Skip this iteration and move to the next one
        print(f"Processing email: {oid}")
        data = read_item(callConnect.container15, oid, oid)

        # Check if the data fetched is a string and convert it to a dictionary
        if isinstance(data, str):
            data = json.loads(data)

        # Update the JSON data with values from the DataFrame row
        await update_json_from_df(data, row)


if __name__ == "__main__":
    import asyncio

    # Run the asynchronous code
    asyncio.run(main())
    
